@@ -17,9 +17,11 @@ use crate::logical_plan::optimizer::{
 use crate::physical_plan::state::ExecutionState;
 #[cfg(any(feature = "parquet", feature = "csv-file"))]
 use crate::prelude::aggregate_scan_projections::agg_projection;
+use crate::prelude::fast_projection::FastProjection;
 use crate::prelude::simplify_expr::SimplifyBooleanRule;
 use crate::utils::combine_predicates_expr;
 use crate::{logical_plan::FETCH_ROWS, prelude::*};
+use polars_io::csv::NullValues;
 
 #[derive(Clone)]
 #[cfg(feature = "csv-file")]
@@ -35,6 +37,7 @@ pub struct LazyCsvReader<'a> {
     schema_overwrite: Option<&'a Schema>,
     low_memory: bool,
     comment_char: Option<u8>,
+    null_values: Option<NullValues>,
 }
 
 #[cfg(feature = "csv-file")]
@@ -52,6 +55,7 @@ impl<'a> LazyCsvReader<'a> {
             schema_overwrite: None,
             low_memory: false,
             comment_char: None,
+            null_values: None,
         }
     }
 
@@ -105,6 +109,12 @@ impl<'a> LazyCsvReader<'a> {
         self
     }
 
+    /// Set values that will be interpreted as missing/ null.
+    pub fn with_null_values(mut self, null_values: Option<NullValues>) -> Self {
+        self.null_values = null_values;
+        self
+    }
+
     /// Cache the DataFrame after reading.
     pub fn with_cache(mut self, cache: bool) -> Self {
         self.cache = cache;
@@ -130,6 +140,7 @@ impl<'a> LazyCsvReader<'a> {
             self.schema_overwrite,
             self.low_memory,
             self.comment_char,
+            self.null_values,
         )
         .build()
         .into();
@@ -528,6 +539,8 @@ impl LazyFrame {
             rules.push(Box::new(opt));
         }
 
+        rules.push(Box::new(FastProjection {}));
+
         let opt = StackOptimizer {};
         lp_top = opt.optimize_loop(&mut rules, expr_arena, lp_arena, lp_top);
 
@@ -732,6 +745,12 @@ impl LazyFrame {
         self.join(other, vec![left_on], vec![right_on], JoinType::Inner)
     }
 
+    /// Creates the cartesian product from both frames, preserves the order of the left keys.
+    #[cfg(feature = "cross_join")]
+    pub fn cross_join(self, other: LazyFrame) -> LazyFrame {
+        self.join(other, vec![], vec![], JoinType::Cross)
+    }
+
     /// Generic join function that can join on multiple columns.
     ///
     /// # Example
@@ -1473,7 +1492,7 @@ mod test {
     }
 
     #[test]
-    fn test_lazy_query_6() {
+    fn test_lazy_query_6() -> Result<()> {
         let df = df! {
             "uid" => [0, 0, 0, 1, 1, 1],
             "day" => [1, 2, 4, 1, 2, 3],
@@ -1486,39 +1505,13 @@ mod test {
             .groupby(vec![col("uid")])
             // a double aggregation expression.
             .agg(vec![pearson_corr(col("day"), col("cumcases")).pow(2.0)])
+            .sort("uid", false)
             .collect()
             .unwrap();
-        dbg!(out);
-    }
-
-    #[test]
-    #[cfg(feature = "temporal")]
-    fn test_lazy_query_7() {
-        let date = NaiveDate::from_ymd(2021, 3, 5);
-        let dates = vec![
-            NaiveDateTime::new(date, NaiveTime::from_hms(12, 0, 0)),
-            NaiveDateTime::new(date, NaiveTime::from_hms(12, 1, 0)),
-            NaiveDateTime::new(date, NaiveTime::from_hms(12, 2, 0)),
-            NaiveDateTime::new(date, NaiveTime::from_hms(12, 3, 0)),
-            NaiveDateTime::new(date, NaiveTime::from_hms(12, 4, 0)),
-            NaiveDateTime::new(date, NaiveTime::from_hms(12, 5, 0)),
-        ];
-        let data = vec![Some(1.), Some(2.), Some(3.), Some(4.), None, None];
-        let df = DataFrame::new(vec![
-            Date64Chunked::new_from_naive_datetime("date", &*dates).into(),
-            Series::new("data", data),
-        ])
-        .unwrap();
-        // this tests if predicate pushdown not interferes with the shift data.
-        let out = df
-            .lazy()
-            .with_column(col("data").shift(-1).alias("output"))
-            .with_column(col("output").shift(2).alias("shifted"))
-            .filter(col("date").gt(lit(NaiveDateTime::new(date, NaiveTime::from_hms(12, 2, 0)))))
-            .collect()
-            .unwrap();
-        let a = out.column(&"shifted").unwrap().sum::<f64>().unwrap() - 7.0;
-        assert!(a < 0.01 && a > -0.01);
+        let s = out.column("pearson_corr")?.f64()?;
+        assert!((s.get(0).unwrap() - 0.994360902255639).abs() < 0.000001);
+        assert!((s.get(1).unwrap() - 0.9552238805970149).abs() < 0.000001);
+        Ok(())
     }
 
     #[test]
@@ -1552,6 +1545,76 @@ mod test {
         Ok(())
     }
 
+    #[test]
+    fn test_lazy_query_9() -> Result<()> {
+        // https://github.com/pola-rs/polars/issues/958
+        let cities = df![
+            "Cities.City"=> ["Moscow", "Berlin", "Paris","Hamburg", "Lyon", "Novosibirsk"],
+            "Cities.Population"=> [11.92, 3.645, 2.161, 1.841, 0.513, 1.511],
+            "Cities.Country"=> ["Russia", "Germany", "France", "Germany", "France", "Russia"]
+        ]?;
+
+        let sales = df![
+                   "Sales.City"=> ["Moscow", "Berlin", "Paris", "Moscow", "Berlin", "Paris", "Moscow", "Berlin", "Paris"],
+        "Sales.Item"=> ["Item A", "Item A","Item A",
+                       "Item B", "Item B","Item B",
+                       "Item C", "Item C","Item C"],
+        "Sales.Amount"=> [200, 180, 100,
+                        3, 30, 20,
+                        90, 130, 125]
+            ]?;
+
+        let out = sales
+            .lazy()
+            .join(
+                cities.lazy(),
+                vec![col("Sales.City")],
+                vec![col("Cities.City")],
+                JoinType::Inner,
+            )
+            .groupby(vec![col("Cities.Country")])
+            .agg(vec![col("Sales.Amount").sum().alias("sum")])
+            .sort("sum", false)
+            .collect()?;
+        let vals = out
+            .column("sum")?
+            .i32()?
+            .into_no_null_iter()
+            .collect::<Vec<_>>();
+        assert_eq!(vals, &[245, 293, 340]);
+        Ok(())
+    }
+
+    #[test]
+    #[cfg(feature = "temporal")]
+    fn test_lazy_query_7() {
+        let date = NaiveDate::from_ymd(2021, 3, 5);
+        let dates = vec![
+            NaiveDateTime::new(date, NaiveTime::from_hms(12, 0, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(12, 1, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(12, 2, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(12, 3, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(12, 4, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(12, 5, 0)),
+        ];
+        let data = vec![Some(1.), Some(2.), Some(3.), Some(4.), None, None];
+        let df = DataFrame::new(vec![
+            Date64Chunked::new_from_naive_datetime("date", &*dates).into(),
+            Series::new("data", data),
+        ])
+        .unwrap();
+        // this tests if predicate pushdown not interferes with the shift data.
+        let out = df
+            .lazy()
+            .with_column(col("data").shift(-1).alias("output"))
+            .with_column(col("output").shift(2).alias("shifted"))
+            .filter(col("date").gt(lit(NaiveDateTime::new(date, NaiveTime::from_hms(12, 2, 0)))))
+            .collect()
+            .unwrap();
+        let a = out.column(&"shifted").unwrap().sum::<f64>().unwrap() - 7.0;
+        assert!(a < 0.01 && a > -0.01);
+    }
+
     #[test]
     fn test_lazy_shift_and_fill_all() {
         let data = &[1, 2, 3];
@@ -2333,4 +2396,43 @@ mod test {
         assert_eq!(a.null_count(), 1);
         Ok(())
     }
+
+    #[cfg(feature = "cross_join")]
+    #[test]
+    fn test_cross_join() -> Result<()> {
+        let df1 = df![
+            "a" => ["a", "b", "a"],
+            "b" => [Some(1), None, None]
+        ]?;
+
+        let df2 = df![
+            "a" => [1, 2],
+            "b" => [None, Some(12)]
+        ]?;
+
+        let out = df1.lazy().cross_join(df2.lazy()).collect()?;
+        assert_eq!(out.shape(), (6, 4));
+        Ok(())
+    }
+
+    #[test]
+    fn test_fold_wildcard() -> Result<()> {
+        let df1 = df![
+        "a" => [1, 2, 3],
+        "b" => [1, 2, 3]
+        ]?;
+
+        let out = df1
+            .lazy()
+            .select(vec![
+                fold_exprs(lit(0), |a, b| Ok(&a + &b), vec![col("*")]).alias("foo")
+            ])
+            .collect()?;
+
+        assert_eq!(
+            Vec::from(out.column("foo")?.i32()?),
+            &[Some(2), Some(4), Some(6)]
+        );
+        Ok(())
+    }
 }
